#!/usr/bin/python
import argparse
import json
import os
import socket
import subprocess

from jinja2 import Template

LS_URL = ("https://download.elasticsearch.org/"
          "logstash/logstash/logstash-1.4.1.tar.gz")
LS_FNAME = os.path.basename(LS_URL)
LS_DIR = "/opt/logstash"
LS_CONF_DIR = "/etc/logstash.d"


def update_config():
    """Update configuration on the system.
    """
    inputs_collectd = get_collectd()
    outputs_es = get_elasticsearch()

    modified = write_file(
        '/etc/logstash.d/20-juju-inputs.conf',
        'inputs.conf.template',
        dict(collectd=inputs_collectd))

    modified = modified or write_file(
        '/etc/logstash.d/70-juju-outputs.conf',
        'outputs.conf.template',
        dict(elasticsearch=outputs_es))

    if modified:
        subprocess.check_output(
            ['service', 'logstash', 'restart'])


def collectd_joined():
    port = subprocess.check_output(
        ['relation-get', 'port', os.environ['JUJU_UNIT_NAME']]).strip()
    if not port:
        port = get_open_port()
    subprocess.check_output(['relation-set', 'port=%d' % port])
    update_config()


def install():
    if not os.path.exists(LS_DIR):
        os.mkdir(LS_DIR)

    ls_tgz = os.path.join(
        os.environ['CHARM_DIR'], 'files', LS_FNAME)

    if not os.path.exists(ls_tgz):
        ls_tgz = os.path.join('/tmp', LS_FNAME)
        subprocess.check_call(['curl', '-o', ls_tgz, LS_URL])
        subprocess.check_call(
            ['md5sum', '-c', os.path.join(
                os.environ['CHARM_DIR'], 'files', 'checksums')])

    subprocess.check_call([
        'tar', '-xzv', '--strip-components=1',
        '-C', "/opt/logstash", '-f', ls_tgz])


##########
# relation helpers
def get_collectd():
    collectd_rels = json.loads(subprocess.check_output(
        ['relation-ids', '--format=json', 'input-collectd'])) or []
    for c in collectd_rels:
        subprocess.check_output([
            'relation-get', '-r', c, 'port', os.environ['JUJU_UNIT_NAME']])


def get_elasticsearch():
    pass


##########
## generic helpers

class AttrDict(dict):

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError(key)


def write_file(path, template, data):
    """Render the named jinja 'template' with the given template 'data' to
    'path'. Return true if the contents of the path changed.
    """
    template = get_template(template)
    content = template.render(**data)
    if os.path.exists(path):
        with open(path) as fh:
            config = fh.read()
            if config == content:
                return False
    with open(path, 'w') as fh:
        fh.write(content)
    return True


def get_template(name):
    template_path = os.path.join(os.environ['CHARM_DIR'], 'files', name)
    with open(template_path) as fh:
        content = fh.read()
    return Template(content)


def get_open_port(host=""):
    """Get an open port on the machine.
    """
    temp_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    temp_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    temp_sock.bind((host, 0))
    port = temp_sock.getsockname()[1]
    temp_sock.close()
    del temp_sock
    return port


def setup_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("--install", action="store_true")
    return parser


def main():
    parser = setup_parser()
    options = parser.parse_args()
    if options.install:
        install()

if __name__ == '__main__':
    main()
